<?php
/*
Plugin Name: Fotherplot
Version: 0.0.6
Plugin URI: http://www.tomfotherby.com/Websites/Fotherplot
Description: A plugin for listing and plotting your data against time using the Google charts API. Originally used to plot my weight by using a custom field called "weight" in posts but useful for all sorts of other value tracking (e.g. gym-time, money, hours in office, mood rating, etc, etc).
Author: Tom Fotherby
Author URI: http://www.tomfotherby.com
*/

/*  Copyright 2009  Tom Fotherby, (email : fotherplotWPPlugin@tomfotherby.com)

    This program is free software; you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

function plotDataAgainstTime($customFieldName = "",$showGraph=1,$showList=1)
{

 if ($customFieldName == "")
    $customFieldName = get_option('fotherplot_customFieldName');
 if ($customFieldName == "") {
    $returnText .= "<div class='error'><b>Error! Can't draw chart</b> - <i>No custom field registered with the fotherplot plugin</i>. Please set the name of the custom field you want to chart (in the fotherplot options menu on the Wordpress dashboard).</div>";
    return "";
 }
 
 // Query the database for the posts that contain this custom field
 global $wpdb;
 $query_string = "
 SELECT *
 FROM $wpdb->posts
 LEFT JOIN $wpdb->postmeta ON ($wpdb->posts.ID = $wpdb->postmeta.post_id)
 WHERE $wpdb->postmeta.meta_key = '$customFieldName'
 AND $wpdb->posts.post_status = 'publish'
 AND $wpdb->posts.post_type = 'post'
 ORDER BY $wpdb->posts.post_date ASC
 ";

 // List the posts
 $series_posts = $wpdb->get_results($query_string, OBJECT);

 // Initilise variables needed
 $outputText = "";
 $returnText = "";
 $googleChartDataY = array();
 $googleChartLabsY = "";
 $numOfYLabs       = 5;
 $googleChartDataX = array();
 $googleChartLabsX = "";
 $numOfXLabs       = 5;
 $firstDate = "";
 $numOfPoints = sizeof($series_posts);

 if ($series_posts):
   foreach ($series_posts as $post):
   	   if ($firstDate == "")
	      $firstDate = strtotime($post->post_date);

     	   $dayDiff = round((strtotime($post->post_date)-$firstDate) / (60 * 60 * 24));
	   $dataVal = $post->meta_value;

	   if (!is_numeric($dataVal)) {
	      $returnText .= "<div class='error'><b>Error! Can't draw chart</b> - non numeric value \"<i>$dataVal</i>\" used in post on <i>$post->post_date</i>. Please change this value so it is numeric or change the custom field to one you know to be numeric.</div>";
	      return 1;
	   }

	   array_push($googleChartDataY,$dataVal);
	   array_push($googleChartDataX,$dayDiff);
   	   $outputText = "<li><a href=\"" . get_permalink($post->ID) . "\">" . date("F j, Y",strtotime($post->post_date)) . "</a>: <b>{$dataVal}</b></li>" . $outputText;
   endforeach;

   // Y Axis Labels - Data values. Start at min, end at Max, distribute evenly in between
   $stepSize = (max($googleChartDataY)-min($googleChartDataY))/($numOfYLabs-1);
   for ($i=min($googleChartDataY); $i<=max($googleChartDataY); $i+=$stepSize) {
        $googleChartLabsY .= "|$i";
   }

   // X Axis Labels - Date values. Start at oldest, end at Newest, distribute evenly in between
   $stepSize = max($googleChartDataX)/($numOfXLabs-1);
   for ($i=0; $i<=max($googleChartDataX); $i+=$stepSize) {
        $googleChartLabsX .= "|".date("d/M/y",strtotime("+ ".(int)$i." days",$firstDate));
   }


   $chartType    = "&cht=lxy";          // Line chart with x,y co-ordinates
   $chartSize    = "&chs=450x300";
   $chartMargins = "&chma=10,10,10,10";
   $axisScale    = "&chds=0,".max($googleChartDataX).",".min($googleChartDataY).",".max($googleChartDataY);
   $axisGrid     = "&chg=0,5";
   //$axisLabels   = "&chxt=y&chxr=0,$minDataVal,$maxDataVal";  // Specify Range
   $axisLabels   = "&chxt=y,x&chxl=0:{$googleChartLabsY}|1:{$googleChartLabsX}";
   //$axisLabels   = "&chxt=x,y,x,y&chxl=0:{$googleChartLabsX}|1:{$googleChartLabsY}|2:||Time scale||3:||Data|";
   $dataLabels   = "&chm=D,FF0000,0,-1,1";    // Display a red line
   $dataLabels  .= "|o,000000,0,-1,4";        // Display a dot at each plot
   $dataLabels  .= "|N,000000,0,-1,9";        // Display the X-axis value for each point
   //$dataLabels  .= "|N*f0x*,000000,0,-1,9"; // Display the 'Y' value for each point
   $dataPoints   = "&chd=t:".join(",",$googleChartDataX)."|".join(",",$googleChartDataY);

   if ($showGraph)
      $returnText .= "<h4>$customFieldName - Chart</h4><center><img src=\"http://chart.apis.google.com/chart?{$chartType}{$chartSize}{$chartMargins}{$axisScale}{$axisGrid}{$axisLabels}{$dataLabels}{$dataPoints}\" /></center>";

   if ($showList)
      $returnText .= "<h4>$customFieldName - Data values</h4><ul>$outputText</ul>";
 endif;

 if ($numOfPoints < 2)
   $returnText .= "<p>Fotherplot can't yet draw a chart because you have $numOfPoints posts that use custom field \"<b>$customFieldName</b>\". You need a minimum of 2.</p>";

 return $returnText;
}

// fotherplot_options() displays the page content for the Fotherplot Options dashboard submenu
function fotherplot_options()
{
	// variables for the field and option names 
	$opt_name        = 'fotherplot_customFieldName';
	$data_field_name = 'fotherplot_customFieldName';

	// Read in existing option value from database
	$opt_val = get_option($opt_name);

        if($_POST['fotherplot_save'])
	{
		// Read their posted value
		$opt_val = $_POST[ $data_field_name ];

		// Save the posted value in the database
                update_option($opt_name,$opt_val);

                echo '<div class="updated"><p>Custom Field name saved successfully.</p></div>';
        }

        ?>
        <div class="wrap">
        <h2>Fotherplot Options</h2>
	<p>Fotherplot charts the value of a custom field over time using the "published-on" date from any posts that have it set.</p>
        <form method="post" id="fotherplot_options">
                <fieldset class="options">
                <table cellspacing="2" cellpadding="5" class="editform">
                        <tr valign="top">
                                <th scope="row">Specify the custom field that you want to chart:</th>
                                <td>
				<select name="<?php echo $data_field_name;?>">
				<?php
				$numOfOptions = 0;
				// Gather all custom field names (then ignore the ones that start with "_")
				global $wpdb;
				$query_string = "SELECT DISTINCT meta_key FROM $wpdb->postmeta";
				$customFieldList = $wpdb->get_results($query_string, OBJECT);
				foreach ($customFieldList as $cf):
					$cfName = $cf->meta_key;
					if ($cfName[0] != "_") {
					   $numOfOptions++;
					   if ($cfName==$opt_val)
	      	 			      echo "<option value=\"$cfName\" selected='selected'>$cfName</option>";
					   else
	      	 			      echo "<option value=\"$cfName\">$cfName</option>";
					}
				endforeach;
				if ($numOfOptions == 0)
				   echo "<div class='error'><b>Error! Can't draw chart</b> - You have no custom fields in use. Please write a couple of posts and create a new custom field as you do so.</div>";
				?>
				</select>
				</td>
                        </tr>
                </table>
                <p class="submit"><input type="submit" name="fotherplot_save" value="Save" /></p>
                </fieldset>
        </form>
	The output of fotherplot is shown below. For instructions of how to include this chart in pages or posts see the <a href="http://wordpress.org/extend/plugins/fotherplot/faq/">fotherplot faqs</a>.
        <?php 
	// Call the function that draws the chart
	plotDataAgainstTime();
	?>
        </div>
	<?php
}

// Hook for adding Wordpress Admin menus
add_action('admin_menu','fotherplot_adminmenu');

// Action function for above hook - i.e. Add page link to Wordpress Admin Menu
function fotherplot_adminmenu()
{
	// Add a new dashboard submenu under Options:
	//  Format: add_options_page(page_title, menu_title, access_level/capability, file, [function]);
        add_options_page('Fotherplot Options', 'Fotherplot', 9, 'fotherplot.php', 'fotherplot_options');
}

// Hook for adding chart anchor text
add_filter('the_content', 'filterFotherplot');

// Action function for above hook - i.e. Add chart where '[[fotherplot]]' text is embedded
function filterFotherplot($content) 
{
  if(preg_match("[fotherplot]",$content))
  {
    $posOfFilter = strpos($content,'[[fotherplot]]');
    if ($posOfFilter !== FALSE)
    {
      $contentStart = substr($content,0,$posOfFilter); 
      $contentEnd   = substr($content,$posOfFilter+strlen('[[fotherplot]]'),strlen($content));
      $content = $contentStart . plotDataAgainstTime("",1,0) . $contentEnd;
    }
  }

  return $content;
}

?>